﻿// Compiler options:  /unsafe /optimize /debug- /target:library /out:"C:\Program Files\Paint.NET\Effects\TRsFireWorks.dll"
using System;
using System.Text;
using System.Windows;
using System.Reflection;
using System.Windows.Forms;
using PaintDotNet;
using PaintDotNet.Effects;
using PaintDotNet.IndirectUI;
using PaintDotNet.PropertySystem;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("TRsFireWorks")]
[assembly: AssemblyDescription("TRsFireWorks Plugin for Paint.NET. (Compiled by Code Lab v1.8)")]
[assembly: AssemblyConfiguration("")]
[assembly: AssemblyCompany("TechnoRobbo")]
[assembly: AssemblyProduct("TRsFireWorksPlugin")]
[assembly: AssemblyCopyright("Copyright 2013 © TechnoRobbo")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]
[assembly: ComVisible(false)]
[assembly: AssemblyVersion("1.2.0")]

namespace TRsFireWorks
{
    public class PluginSupportInfo : IPluginSupportInfo
    {
        public string Author
        {
            get
            {
                return "TechnoRobbo";
            }
        }
        public string Copyright
        {
            get
            {
                return ((AssemblyCopyrightAttribute)base.GetType().Assembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false)[0]).Copyright;
            }
        }

        public string DisplayName
        {
            get
            {
                return ((AssemblyProductAttribute)base.GetType().Assembly.GetCustomAttributes(typeof(AssemblyProductAttribute), false)[0]).Product;
            }
        }

        public Version Version
        {
            get
            {
                return base.GetType().Assembly.GetName().Version;
            }
        }

        public Uri WebsiteUri
        {
            get
            {
                return new Uri("http://www.technorobbo.com");
            }
        }
    }

    [PluginSupportInfo(typeof(PluginSupportInfo), DisplayName = "TR's FireWorks")]
    public class TRsFireWorksEffectPlugin : PropertyBasedEffect
    {
        public static string StaticName
        {
            get
            {
                return "TR's FireWorks";
            }
        }

        public static Bitmap StaticImage { get { return TRsFireWorks.Properties.Resources.icon1; } }

        public TRsFireWorksEffectPlugin()
            : base(StaticName, StaticImage, SubmenuNames.Render, EffectFlags.Configurable)
        {
            instanceSeed = unchecked((int)DateTime.Now.Ticks);
        }

        public enum PropertyNames
        {
            Amount1,
            Amount2,
            Amount3,
            Amount4,
            Amount5,
            Amount6,
            Amount7,
            Amount8,
            Amount9
        }

        [ThreadStatic]
        private static Random RandomNumber;

        private int randomSeed;
        private int instanceSeed;


        protected override PropertyCollection OnCreatePropertyCollection()
        {
            List<Property> props = new List<Property>();

            props.Add(new DoubleVectorProperty(PropertyNames.Amount1, Pair.Create(0.0, 0.0), Pair.Create(-1.0, -1.0), Pair.Create(+1.0, +1.0)));
            props.Add(new Int32Property(PropertyNames.Amount2, 100, 100, 1000));
            props.Add(new Int32Property(PropertyNames.Amount3, 0, 0, 255));
            props.Add(new Int32Property(PropertyNames.Amount4, 1, 0, 15));
            props.Add(new Int32Property(PropertyNames.Amount8, 32, 32, 128));
            props.Add(new DoubleProperty(PropertyNames.Amount9, 0.25, 0.05, 0.95));
            props.Add(new Int32Property(PropertyNames.Amount5, ColorBgra.ToOpaqueInt32(ColorBgra.FromBgra(255, 0, 0, 255)), 0, 0xffffff));
            props.Add(new Int32Property(PropertyNames.Amount6, ColorBgra.ToOpaqueInt32(ColorBgra.FromBgra(0,0,255, 255)), 0, 0xffffff));
            props.Add(new Int32Property(PropertyNames.Amount7, ColorBgra.ToOpaqueInt32(ColorBgra.FromBgra(128, 255, 255, 255)), 0, 0xffffff));


            return new PropertyCollection(props);
        }

        protected override ControlInfo OnCreateConfigUI(PropertyCollection props)
        {
            ControlInfo configUI = CreateDefaultConfigUI(props);

            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.DisplayName, "Location");
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.SliderSmallChangeX, 0.05);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.SliderLargeChangeX, 0.25);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.UpDownIncrementX, 0.01);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.SliderSmallChangeY, 0.05);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.SliderLargeChangeY, 0.25);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.UpDownIncrementY, 0.01);
            Surface sourceSurface1 = this.EnvironmentParameters.SourceSurface;
            Bitmap bitmap1 = sourceSurface1.CreateAliasedBitmap();
            ImageResource imageResource1 = ImageResource.FromImage(bitmap1);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.StaticImageUnderlay, imageResource1);
            configUI.SetPropertyControlValue(PropertyNames.Amount2, ControlInfoPropertyNames.DisplayName, "Time");
            configUI.SetPropertyControlValue(PropertyNames.Amount3, ControlInfoPropertyNames.DisplayName, string.Empty);
            configUI.SetPropertyControlType(PropertyNames.Amount3, PropertyControlType.IncrementButton);
            configUI.SetPropertyControlValue(PropertyNames.Amount3, ControlInfoPropertyNames.ButtonText, "Re - Seed");
            configUI.SetPropertyControlValue(PropertyNames.Amount4, ControlInfoPropertyNames.DisplayName, "Width");
            configUI.SetPropertyControlValue(PropertyNames.Amount5, ControlInfoPropertyNames.DisplayName, "Center Color");
            configUI.SetPropertyControlType(PropertyNames.Amount5, PropertyControlType.ColorWheel);
            configUI.SetPropertyControlValue(PropertyNames.Amount6, ControlInfoPropertyNames.DisplayName, "Outer Color");
            configUI.SetPropertyControlType(PropertyNames.Amount6, PropertyControlType.ColorWheel);
            configUI.SetPropertyControlValue(PropertyNames.Amount7, ControlInfoPropertyNames.DisplayName, "Ember Color");
            configUI.SetPropertyControlType(PropertyNames.Amount7, PropertyControlType.ColorWheel);
            configUI.SetPropertyControlValue(PropertyNames.Amount8, ControlInfoPropertyNames.DisplayName, "Ember Count");
            configUI.SetPropertyControlValue(PropertyNames.Amount9, ControlInfoPropertyNames.DisplayName, "Gravity");
            configUI.SetPropertyControlValue(PropertyNames.Amount9, ControlInfoPropertyNames.SliderLargeChange, 0.25);
            configUI.SetPropertyControlValue(PropertyNames.Amount9, ControlInfoPropertyNames.SliderSmallChange, 0.05);
            configUI.SetPropertyControlValue(PropertyNames.Amount9, ControlInfoPropertyNames.UpDownIncrement, 0.01);
            return configUI;
        }
        bool Semaph = true; // Semaphore
        protected override void OnSetRenderInfo(PropertyBasedEffectConfigToken newToken, RenderArgs dstArgs, RenderArgs srcArgs)
        {
            this.Amount1 = newToken.GetProperty<DoubleVectorProperty>(PropertyNames.Amount1).Value;
            this.Amount2 = newToken.GetProperty<Int32Property>(PropertyNames.Amount2).Value;
            this.Amount3 = (byte)newToken.GetProperty<Int32Property>(PropertyNames.Amount3).Value;
            randomSeed = Amount3;
            this.Amount4 = newToken.GetProperty<Int32Property>(PropertyNames.Amount4).Value;
            this.Amount5 = ColorBgra.FromOpaqueInt32(newToken.GetProperty<Int32Property>(PropertyNames.Amount5).Value);
            this.Amount6 = ColorBgra.FromOpaqueInt32(newToken.GetProperty<Int32Property>(PropertyNames.Amount6).Value);
            this.Amount7 = ColorBgra.FromOpaqueInt32(newToken.GetProperty<Int32Property>(PropertyNames.Amount7).Value);
            this.Amount8 = newToken.GetProperty<Int32Property>(PropertyNames.Amount8).Value;
            this.Amount9 = newToken.GetProperty<DoubleProperty>(PropertyNames.Amount9).Value;
            Semaph = true;
            base.OnSetRenderInfo(newToken, dstArgs, srcArgs);
        }


        protected override void OnCustomizeConfigUIWindowProperties(PropertyCollection props)
        {
            // Change the effect's window title
            Version version = Assembly.GetExecutingAssembly().GetName().Version;
            props[ControlInfoPropertyNames.WindowTitle].Value = StaticName + " ver. " + version.Major + "." + version.Minor + "." + version.Build; ;
            base.OnCustomizeConfigUIWindowProperties(props);
        }

        protected override unsafe void OnRender(Rectangle[] rois, int startIndex, int length)
        {
            if (length == 0) return;
            RandomNumber = GetRandomNumberGenerator(rois, startIndex);
            for (int i = startIndex; i < startIndex + length; ++i)
            {
                Render(DstArgs.Surface, SrcArgs.Surface, rois[i]);
            }
        }

        private Random GetRandomNumberGenerator(Rectangle[] rois, int startIndex)
        {
            Rectangle roi = rois[startIndex];
            return new Random(instanceSeed ^ (randomSeed << 16) ^ (roi.X << 8) ^ roi.Y);
        }

        #region User Entered Code
        // Submenu: Render
        // Name: TR's FireWorks
        // Title: TR's FireWorks - v1.1
        // Author: TechnoRobbo
        // URL: http://www.technorobbo.com

        #region UICode
        Pair<double, double> Amount1 = Pair.Create(0.0, 0.0); // Location
        int Amount2 = 100; // [100,1000] Time
        byte Amount3 = 0; // [255] Re - Seed
        int Amount4 = 1; // [0,15] Width
        ColorBgra Amount5 = ColorBgra.FromBgr(0, 0, 0); // Start Color
        ColorBgra Amount6 = ColorBgra.FromBgr(0, 0, 0); // End Color
        ColorBgra Amount7 = ColorBgra.FromBgr(0, 0, 0); 
        int Amount8 = 32; // [32,128] Ember Count
        double Amount9 = 0.25;
        #endregion

        int Seed = (int)Guid.NewGuid().GetHashCode();

        void Render(Surface dst, Surface src, Rectangle rect)
        {
            if (Semaph)
            {
                Semaph = false;
                Random rndm = new Random(Seed + Amount3);
                Rectangle selection = EnvironmentParameters.GetSelection(src.Bounds).GetBoundsInt();
                int CenterX = ((selection.Right - selection.Left) / 2) + selection.Left;
                int CenterY = ((selection.Bottom - selection.Top) / 2) + selection.Top;
                System.Drawing.PointF Spark = new System.Drawing.PointF((float)CenterX *
                    (float)Amount1.First + CenterX, (float)CenterY * (float)Amount1.Second + CenterY);

                System.Drawing.PointF[] Sparks = new System.Drawing.PointF[Amount8];
                System.Drawing.PointF[] Dirs = new System.Drawing.PointF[Amount8];
                ColorBgra Primary = Amount5;
                ColorBgra Secondary = Amount6;
                ColorBgra Ember = Amount7;
                int BrushWidth = Amount4;

                dst.CopySurface(src);

                Bitmap bmp = new Bitmap(src.Width, src.Height);
                Graphics g = Graphics.FromImage(bmp);
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                g.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceOver;
                for (int i = 0; i < Amount8; i++)
                {
                    Sparks[i] = Spark;
                    double rad = rndm.NextDouble() * (Math.PI * 2);
                    float dist = (float)rndm.NextDouble() * Amount2 * .75f + Amount2 * .25f;
                    Dirs[i].X = (float)Math.Cos(rad) * dist + Spark.X;
                    Dirs[i].Y = (float)Math.Sin(rad) * dist + Spark.Y;
                }
                for (int tm = 0; tm < Amount2; tm++)
                {
                    float t = (float)tm / Amount2;
                    for (int i = 0; i < Amount8; i++)
                    {
                        Pen p;
                        if (tm == (Amount2 - 1))
                        {
                            p = new Pen(Color.FromArgb(192, Ember.R, Ember.G, Ember.B));
                        }
                        else
                        {
                            p = new Pen(Color.FromArgb((int)(t * 128),
                                (int)(Secondary.R * t + Primary.R - Primary.R * t),
                                (int)(Secondary.G * t + Primary.G - Primary.G * t),
                                (int)(Secondary.B * t + Primary.B - Primary.B * t)));
                        }

                        p.Width = BrushWidth;

                        int x = (int)(Dirs[i].X * t + Sparks[i].X - Sparks[i].X * t);
                        int y = (int)(Dirs[i].Y * t + Sparks[i].Y - Sparks[i].Y * t);
                        g.DrawEllipse(p, new System.Drawing.RectangleF(x - BrushWidth / 2, y - BrushWidth / 2, BrushWidth, BrushWidth));

                        Dirs[i].Y += (float)Amount9;
                        p.Dispose();
                    }


                }
                g.Dispose();
                Graphics gn = new RenderArgs(dst).Graphics;
                gn.DrawImage(bmp, 0, 0);
                gn.Dispose();


                bmp.Dispose();

            }
            else { dst[0, 0] = dst[0, 0]; }
        }


        #endregion
    }
}
